package main

import (
	"bytes"
	"fmt"
	"io"
	"io/ioutil"
	"math/rand"
	"mime/multipart"
	"net/http"
	"net/textproto"
	"os"
	"os/exec"
	"os/signal"
	"strconv"
	"strings"
	"syscall"
	"time"

	"github.com/bwmarrin/discordgo"
	"github.com/ryanuber/columnize"
	"github.com/unixvoid/glogger"
	"gopkg.in/gcfg.v1"
	"gopkg.in/redis.v5"
)

type Config struct {
	Lorebot struct {
		Loglevel       string
		SlackDebug     bool
		BootstrapDelay time.Duration
		AuthFile       string
	}
	Redis struct {
		Host     string
		Password string
	}
}

var (
	config     = Config{}
	rrCylinder int
)

func main() {
	// read in config file
	readConf()

	// initialize the logger with the configured loglevel
	initLogger(config.Lorebot.Loglevel)

	// read in creds
	credentials, _ := ioutil.ReadFile(config.Lorebot.AuthFile)
	lines := strings.Split(string(credentials), "\n")
	auth := lines[0]

	// seed the rr cylinder
	rrCylinder = rand.Intn(6-1) + 1

	// Create a new Discord session using the provided bot token.
	goBot, err := discordgo.New("Bot " + auth)
	if err != nil {
		fmt.Println("error creating Discord session,", err)
		return
	}

	// Register the msgParser func as a callback for message events.
	goBot.AddHandler(msgParser)

	// Open a websocket connection to Discord and begin listening.
	err = goBot.Open()
	if err != nil {
		fmt.Println("error opening connection,", err)
		return
	}

	// Wait here until CTRL-C or other term signal is received.
	fmt.Println("Bot is now running.  Press CTRL-C to exit.")
	sc := make(chan os.Signal, 1)
	signal.Notify(sc, syscall.SIGINT, syscall.SIGTERM, os.Interrupt, os.Kill)
	<-sc

	// Cleanly close down the Discord session.
	goBot.Close()
}

func readConf() {
	// init config file
	err := gcfg.ReadFileInto(&config, "config.gcfg")
	if err != nil {
		panic(fmt.Sprintf("Could not load config.gcfg, error: %s\n", err))
	}
}

func initLogger(logLevel string) {
	// init logger
	if logLevel == "debug" {
		glogger.LogInit(os.Stdout, os.Stdout, os.Stdout, os.Stderr)
	} else if logLevel == "cluster" {
		glogger.LogInit(os.Stdout, os.Stdout, ioutil.Discard, os.Stderr)
	} else if logLevel == "info" {
		glogger.LogInit(os.Stdout, ioutil.Discard, ioutil.Discard, os.Stderr)
	} else {
		glogger.LogInit(ioutil.Discard, ioutil.Discard, ioutil.Discard, os.Stderr)
	}
}

func initRedisConnection() (*redis.Client, error) {
	// init redis connection
	redisClient := redis.NewClient(&redis.Options{
		Addr:     config.Redis.Host,
		Password: config.Redis.Password,
		DB:       0,
	})

	_, redisErr := redisClient.Ping().Result()
	return redisClient, redisErr
}

func msgParser(s *discordgo.Session, m *discordgo.MessageCreate) {
	// exit out if its a blank message or gif/photo
	if len(m.Content) == 0 {
		return
	}

	//// // debug message recieved if its not too long
	//// glogger.Debug.Printf("length of msg is: %d", len(m.Content))
	//// if len(m.Content) <= 150 {
	//// 	glogger.Debug.Printf("Message recieved: %s\n", m.Content)
	//// } else {
	//// 	glogger.Debug.Printf("Message too big to print\n")
	//// }

	// Ignore all messages created by the bot itself
	if m.Author.ID == s.State.User.ID {
		return
	}

	comm := fmt.Sprintf("%c", m.Content[0])
	//glogger.Debug.Printf("comm var: %s", comm)
	if comm == "." {
		if strings.Count(m.Content, " ") == 0 {
			// no spaces, its a static command
			glogger.Debug.Printf("processing static command")
			staticCommandHandler(s, m)
		} else {
			// spaces, its a dynamic command
			glogger.Debug.Printf("processing dynamic command")
			dynamicCommandHandler(s, m)
		}
	}

	// If the message is "ping" reply with "Pong!"
	if m.Content == "ping" {
		s.ChannelMessageSend(m.ChannelID, "fuck off bloam")
	}
}

//
// static functions
//
func staticCommandHandler(s *discordgo.Session, m *discordgo.MessageCreate) {
	// initialize redis connection
	redisClient, err := initRedisConnection()
	if err != nil {
		glogger.Debug.Printf("redis conneciton cannot be made, trying again in %d seconds", config.Lorebot.BootstrapDelay)
		time.Sleep(config.Lorebot.BootstrapDelay * time.Second)
		redisClient, err = initRedisConnection()
		if err != nil {
			glogger.Error.Println("redis connection cannot be made.")
		}
	}

	tmpStr := strings.SplitN(m.Content, " ", 2)
	command := tmpStr[0]
	trimmedPrefix := strings.TrimPrefix(command, ".")

	switch trimmedPrefix {
	case "help":
		helpMsg(s, m)
	case "help2":
		advancedHelpMsg(s, m)
	case "rng":
		rngHandler(s, m, redisClient)
	case "shop":
		shopHandler(s, m, redisClient)
	case "dubs":
		dubsHandler(s, m, redisClient)
	case "lunch":
		lunchHandler(s, m, redisClient)
	case "brb":
		brbHandler(s, m, redisClient)
	case "roll":
		rollHandler(s, m)
	case "rr":
		rrHandler(s, m)
	case "dice":
		diceRollHandler(s, m)
	case "ceelo":
		ceeloRollHandler(s, m)
	case "fortune":
		fortuneHandler(s, m)
	case "kenm":
		kenmHandler(s, m)
	case "showerthoughts":
		showerthoughtsHandler(s, m)
	case "listen":
		s.ChannelMessageSend(m.ChannelID, "*it's what you hearin*")
	case "lorelist":
		listHandler(redisClient, s, m, "lore", "added")
	case "removedlore":
		listHandler(redisClient, s, m, "lore", "removed")
	case "commandlist":
		listHandler(redisClient, s, m, "command", "added")
	case "removedcommands":
		listHandler(redisClient, s, m, "command", "removed")
	case "lorestatus":
		loreStatus(s, m, redisClient)
	}

	// get that dice handler
	if len(m.Content) >= 2 {
		if strings.Contains(m.Content, ".d") {
			trm := strings.Replace(m.Content, ".d", "", -1)

			// check if the rest of the string is an int
			num, err := strconv.Atoi(trm)
			if err == nil {
				//glogger.Debug.Printf("rolling for %d", num)
				dHandler(s, m, num)
			}
		}
	}
}

func helpMsg(s *discordgo.Session, m *discordgo.MessageCreate) {
	help1 := "I am the lore archival bot, I support the following commands:\n" +
		"```" +
		".lore:         view lore\n" +
		".newlore:      add new lore\n" +
		".rmlore:       remove lore\n" +
		".lorelist:     view lore database\n" +
		".lorestatus:   view lore DB status\n" +
		".roll:         checkum\n" +
		".rr:           its high noon\n" +
		".dice:         roll a d6\n" +
		".ceelo:        roll a ceelo hand\n" +
		".listen:       rip the dawg\n" +
		"use '.help2' to see advanced lookup commands." +
		"```"

	s.ChannelMessageSend(m.ChannelID, help1)
}

func advancedHelpMsg(s *discordgo.Session, m *discordgo.MessageCreate) {
	help2 := "```" +
		".created       view creation timestamp\n" +
		".owner:        who created the lore\n" +
		".removed:      view removal timestamp\n" +
		".removedby:    who removed the lore\n" +
		".rek:          rek someone\n" +
		".rip:          f's in the chat\n" +
		".summon:       summon someone\n" +
		".gnu:          dont forget your freedoms\n" +
		".brb:          find Tjance's location" +
		"```"

	s.ChannelMessageSend(m.ChannelID, help2)
}

func rngHandler(s *discordgo.Session, m *discordgo.MessageCreate, redisClient *redis.Client) {
	// aka index:command:added
	newType := fmt.Sprintf("index:lore:added")
	//command, err := botfunc.Query(newType, redisClient)
	command, err := loreQuery(newType, redisClient)
	if err != nil {
		glogger.Error.Println(err)
		println(err)
	}

	// if the array is empty, exit early
	if len(command) <= 0 {
		return
	}

	// pick a random one
	rand.Seed(time.Now().UnixNano())
	rngLore := fmt.Sprint(command[rand.Intn(len(command))])
	glogger.Debug.Printf("rng lore: %s\n", rngLore)

	// get lore content
	rngContent, err := loreCheck(rngLore, "lore", redisClient)
	if err != nil {
		glogger.Error.Printf("%v\n", err)
		s.ChannelMessageSend(m.ChannelID, "beep boop.. broken bot..")
		return
	}
	glogger.Debug.Printf("rng content: %s\n", rngContent)
	rngMsg := fmt.Sprintf("Random lore: `%s`\n%s", rngLore, rngContent)
	s.ChannelMessageSend(m.ChannelID, rngMsg)
}

func shopHandler(s *discordgo.Session, m *discordgo.MessageCreate, redisClient *redis.Client) {
	// data format
	// redis set for the following indexes:
	//   index:dnd:common
	//   index:dnd:uncommon
	//   index:dnd:rare
	//   index:dnd:very_rare
	//   index:dnd:legendary
	//
	// redis hash with the following data
	//   dnd:abracadabrus name x rarity x price x type x

	// item settings
	minCommon := 4
	maxCommon := 6
	minUncommon := 2
	maxUncommon := 4
	minRare := 1
	maxRare := 3
	minVeryRare := 0
	maxVeryRare := 2

	// initialize item table
	var itemTable []string
	itemTable = append(itemTable, "Name | Rarity | Price | Type")

	//
	// commons:
	//
	// get common amount (rand.Intn(max - min) + min
	rand.Seed(time.Now().UnixNano())
	commons := rand.Intn(maxCommon-minCommon) + minCommon
	for n := 0; n < commons; n++ {
		// grab random item
		command, err := loreQuery("index:dnd:common", redisClient)
		if err != nil {
			glogger.Error.Println(err)
		}
		// if the array is empty, exit early
		if len(command) <= 0 {
			return
		}
		// pull random item
		rand.Seed(time.Now().UnixNano())
		rngItem := fmt.Sprint(command[rand.Intn(len(command))])

		// get item details for shop message
		rngItemName, _ := redisClient.HGet(rngItem, "name").Result()
		rngItemRarity, _ := redisClient.HGet(rngItem, "rarity").Result()
		rngItemPrice, _ := redisClient.HGet(rngItem, "price").Result()
		rngItemType, _ := redisClient.HGet(rngItem, "type").Result()

		// put together message
		itemTable = append(itemTable, fmt.Sprintf("%s | %s | %s | %s", rngItemName, rngItemRarity, rngItemPrice, rngItemType))
	}

	//
	// uncommons:
	//
	// get uncommon amount (rand.Intn(max - min) + min
	rand.Seed(time.Now().UnixNano())
	uncommons := rand.Intn(maxUncommon-minUncommon) + minUncommon
	for n := 0; n < uncommons; n++ {
		// grab random item
		command, err := loreQuery("index:dnd:uncommon", redisClient)
		if err != nil {
			glogger.Error.Println(err)
		}
		// if the array is empty, exit early
		if len(command) <= 0 {
			return
		}
		// pull random item
		rand.Seed(time.Now().UnixNano())
		rngItem := fmt.Sprint(command[rand.Intn(len(command))])

		// get item details for shop message
		rngItemName, _ := redisClient.HGet(rngItem, "name").Result()
		rngItemRarity, _ := redisClient.HGet(rngItem, "rarity").Result()
		rngItemPrice, _ := redisClient.HGet(rngItem, "price").Result()
		rngItemType, _ := redisClient.HGet(rngItem, "type").Result()

		// put together message
		itemTable = append(itemTable, fmt.Sprintf("%s | %s | %s | %s", rngItemName, rngItemRarity, rngItemPrice, rngItemType))
	}

	//
	// rare:
	//
	// get rare amount (rand.Intn(max - min) + min
	rand.Seed(time.Now().UnixNano())
	rare := rand.Intn(maxRare-minRare) + minRare
	for n := 0; n < rare; n++ {
		// grab random item
		command, err := loreQuery("index:dnd:rare", redisClient)
		if err != nil {
			glogger.Error.Println(err)
		}
		// if the array is empty, exit early
		if len(command) <= 0 {
			return
		}
		// pull random item
		rand.Seed(time.Now().UnixNano())
		rngItem := fmt.Sprint(command[rand.Intn(len(command))])

		// get item details for shop message
		rngItemName, _ := redisClient.HGet(rngItem, "name").Result()
		rngItemRarity, _ := redisClient.HGet(rngItem, "rarity").Result()
		rngItemPrice, _ := redisClient.HGet(rngItem, "price").Result()
		rngItemType, _ := redisClient.HGet(rngItem, "type").Result()

		// put together message
		itemTable = append(itemTable, fmt.Sprintf("%s | %s | %s | %s", rngItemName, rngItemRarity, rngItemPrice, rngItemType))
	}

	//
	// very rare:
	//
	// get very rare amount (rand.Intn(max - min) + min
	rand.Seed(time.Now().UnixNano())
	veryRare := rand.Intn(maxVeryRare-minVeryRare) + minVeryRare
	for n := 0; n < veryRare; n++ {
		// grab random item
		command, err := loreQuery("index:dnd:very_rare", redisClient)
		if err != nil {
			glogger.Error.Println(err)
		}
		// if the array is empty, exit early
		if len(command) <= 0 {
			return
		}
		// pull random item
		rand.Seed(time.Now().UnixNano())
		rngItem := fmt.Sprint(command[rand.Intn(len(command))])

		// get item details for shop message
		rngItemName, _ := redisClient.HGet(rngItem, "name").Result()
		rngItemRarity, _ := redisClient.HGet(rngItem, "rarity").Result()
		rngItemPrice, _ := redisClient.HGet(rngItem, "price").Result()
		rngItemType, _ := redisClient.HGet(rngItem, "type").Result()

		// put together message
		itemTable = append(itemTable, fmt.Sprintf("%s | %s | %s | %s", rngItemName, rngItemRarity, rngItemPrice, rngItemType))
	}

	// put together formatted discord message
	bigMsg := "```\n"
	bigMsg = bigMsg + columnize.SimpleFormat(itemTable)
	bigMsg = bigMsg + "\n```"
	s.ChannelMessageSend(m.ChannelID, bigMsg)
}

func dubsHandler(s *discordgo.Session, m *discordgo.MessageCreate, redisClient *redis.Client) {
	// aka index:command:added
	newType := fmt.Sprintf("index:dubs")

	command, err := loreQuery(newType, redisClient)
	if err != nil {
		glogger.Error.Println(err)
	}

	// if the array is empty, exit early
	if len(command) <= 0 {
		return
	}

	// roll a number
	rollHandler(s, m)

	// pick a random one
	rand.Seed(time.Now().UnixNano())
	rngDubs := fmt.Sprint(command[rand.Intn(len(command))])
	s.ChannelMessageSend(m.ChannelID, rngDubs)

	// roll after we've proccessed dubs
	// you already know tho..
	//re := regexp.MustCompile(`taylor|swift|2725225`)
	//if re.MatchString(rngDubs) {
	//	rndInt := 1 + rand.Intn(10-1)
	//	rndStr := fmt.Sprintf("%06d", rand.Int63n(1e6))
	//	finStr := fmt.Sprintf("%s%d%d", rndStr, rndInt, rndInt)
	//	rtm.SendMessage(rtm.NewOutgoingMessage(finStr, ev.Channel))
	//} else {
	//	rollhandler(rtm, ev)
	//}
}

func lunchHandler(s *discordgo.Session, m *discordgo.MessageCreate, redisClient *redis.Client) {
	// aka index:command:added
	newType := fmt.Sprintf("index:lunch")

	command, err := loreQuery(newType, redisClient)
	if err != nil {
		glogger.Error.Println(err)
	}

	// if the array is empty, exit early
	if len(command) <= 0 {
		return
	}

	// pick a random one
	rand.Seed(time.Now().UnixNano())
	rngDubs := fmt.Sprint(command[rand.Intn(len(command))])
	s.ChannelMessageSend(m.ChannelID, rngDubs)
}

func brbHandler(s *discordgo.Session, m *discordgo.MessageCreate, redisClient *redis.Client) {
	// aka index:command:added
	newType := fmt.Sprintf("index:brb")

	command, err := loreQuery(newType, redisClient)
	if err != nil {
		glogger.Error.Println(err)
	}

	// if the array is empty, exit early
	if len(command) <= 0 {
		return
	}

	// pick a random one
	rand.Seed(time.Now().UnixNano())
	rngDubs := fmt.Sprint(command[rand.Intn(len(command))])
	s.ChannelMessageSend(m.ChannelID, rngDubs)
}

func rollHandler(s *discordgo.Session, m *discordgo.MessageCreate) {
	//fmt.Print(rand.Intn(100))
	rngStr := fmt.Sprintf("%08d", rand.Int63n(1e8))
	s.ChannelMessageSend(m.ChannelID, rngStr)
}

func rrHandler(s *discordgo.Session, m *discordgo.MessageCreate) {
	// decriment counter
	rrCylinder = rrCylinder - 1

	// check if we are at 0
	if rrCylinder == 0 {
		// you've been hit by, you've been struck by a smooth bore
		s.ChannelMessageSend(m.ChannelID, "*BANG*")
		s.ChannelMessageSend(m.ChannelID, "https://i.imgur.com/9jWcCyt.gifv")
		rrCylinder = rand.Intn(6-1) + 1
	} else {
		s.ChannelMessageSend(m.ChannelID, "*click*")
	}
}

func diceRollHandler(s *discordgo.Session, m *discordgo.MessageCreate) {
	//fmt.Print(rand.Intn(100))
	rand.Seed(time.Now().UnixNano())
	s.ChannelMessageSend(m.ChannelID, fmt.Sprintf("%d", rand.Intn(6)+1))
}

func ceeloRollHandler(s *discordgo.Session, m *discordgo.MessageCreate) {
	rand.Seed(time.Now().UnixNano())

	// set condition
	hand := false

	for !hand {
		roll0 := rand.Intn(6) + 1
		roll1 := rand.Intn(6) + 1
		roll2 := rand.Intn(6) + 1

		s.ChannelMessageSend(m.ChannelID, fmt.Sprintf("%d, %d, %d", roll0, roll1, roll2))

		// check for pair
		if (roll0 == roll1) || (roll0 == roll2) || (roll1 == roll2) {
			hand = true
		}
		a := fmt.Sprintf("%d%d%d", roll0, roll1, roll2)
		// check for instant win
		if strings.Contains(a, "4") && strings.Contains(a, "5") && strings.Contains(a, "6") {
			hand = true
		}
		// check for instant loss
		if strings.Contains(a, "1") && strings.Contains(a, "2") && strings.Contains(a, "3") {
			hand = true
		}
		time.Sleep(1000 * time.Millisecond)
	}
}

func fortuneHandler(s *discordgo.Session, m *discordgo.MessageCreate) {
	fortune, err := exec.Command("fortune", "-e", "fortunes").CombinedOutput()
	if err != nil {
		fmt.Printf("%v\n", err)
		s.ChannelMessageSend(m.ChannelID, "Fortune command is broken, contact HFIC")
		return
	}

	s.ChannelMessageSend(m.ChannelID, fmt.Sprintf("%s", fortune))
}

func kenmHandler(s *discordgo.Session, m *discordgo.MessageCreate) {
	fortune, err := exec.Command("fortune", "-e", "kenm").CombinedOutput()
	if err != nil {
		fmt.Printf("%v\n", err)
		s.ChannelMessageSend(m.ChannelID, "kenm command is broken, contact HFIC")
		return
	}

	s.ChannelMessageSend(m.ChannelID, fmt.Sprintf("%s", fortune))
}

func showerthoughtsHandler(s *discordgo.Session, m *discordgo.MessageCreate) {
	fortune, err := exec.Command("fortune", "-e", "showerthoughts").CombinedOutput()
	if err != nil {
		fmt.Printf("%v\n", err)
		s.ChannelMessageSend(m.ChannelID, "showerthoughts command is broken, contact HFIC")
		return
	}

	s.ChannelMessageSend(m.ChannelID, fmt.Sprintf("%s", fortune))
}

func listHandler(redisClient *redis.Client, s *discordgo.Session, m *discordgo.MessageCreate, listType, listAttr string) {
	// aka index:command:added
	newType := fmt.Sprintf("%s:%s:%s", "index", listType, listAttr)
	//command, err := botfunc.Query(newType, redisClient)
	command, err := loreQuery(newType, redisClient)
	if err != nil {
		glogger.Error.Println(err)
		println(err)
	}
	fCommand := strings.Join(command, "\n")

	// initialize buffer
	bodyBuf := &bytes.Buffer{}
	bodyWriter := multipart.NewWriter(bodyBuf)
	// create header and body. filename must be set
	mh := make(textproto.MIMEHeader)
	mh.Set("Content-Type", "application/octet-stream")
	mh.Set("Content-Disposition", "form-data; name=\"file\"; filename=\" \"")
	// write body with mime header
	partWriter, _ := bodyWriter.CreatePart(mh)
	io.Copy(partWriter, bytes.NewBufferString(fCommand))
	contentType := bodyWriter.FormDataContentType()
	bodyWriter.Close()
	// set content type to multipart/form-data, required for bitnuke.io
	h := make(textproto.MIMEHeader)
	h.Set("Content-Type", "multipart/form-data")
	bodyWriter.CreatePart(h)
	bodyWriter.Close()

	// POST to bitnuke.io
	resp, err := http.Post("https://bitnuke.io/upload", contentType, bodyBuf)
	if err != nil {
		println(err)
	}
	defer resp.Body.Close()
	resp_body, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		println(err)
	}
	// format response to be link
	fResponse := fmt.Sprintf("https://bitnuke.io/%s", resp_body[:])
	// print to channel
	//rtm.SendMessage(rtm.NewOutgoingMessage(fResponse, ev.Channel))
	s.ChannelMessageSend(m.ChannelID, fResponse)
}

func loreStatus(s *discordgo.Session, m *discordgo.MessageCreate, redisClient *redis.Client) {
	command, _ := loreQuery("index:lore:added", redisClient)
	loreammount := len(command)
	status := fmt.Sprintf("the lore db has %d entries", loreammount)
	s.ChannelMessageSend(m.ChannelID, status)
}

//
// dynamic functions
//
func dynamicCommandHandler(s *discordgo.Session, m *discordgo.MessageCreate) {
	// initialize redis connection
	redisClient, err := initRedisConnection()
	if err != nil {
		glogger.Debug.Printf("redis conneciton cannot be made, trying again in %d seconds", config.Lorebot.BootstrapDelay)
		time.Sleep(config.Lorebot.BootstrapDelay * time.Second)
		redisClient, err = initRedisConnection()
		if err != nil {
			glogger.Error.Println("redis connection cannot be made.")
		}
	}

	message := m.Content
	if strings.Count(message, " ") < 1 {
		glogger.Debug.Println("not proper syntax")
	}

	// verify syntax
	tmpStr := strings.SplitN(message, " ", 2)
	command, content := tmpStr[0], tmpStr[1]
	trimmedPrefix := strings.TrimPrefix(command, ".")

	switch trimmedPrefix {
	case "lore":
		tmp := strings.SplitN(string(m.Content), " ", 2)
		contentHandler(s, m, tmp[1], "lore", redisClient)
	case "newlore":
		if strings.Contains(content, " ") {
			newHandler(s, m, content, "lore", redisClient)
		} else {
			s.ChannelMessageSend(m.ChannelID, "not proper syntax")
			return
		}
	case "newdubs":
		tmp := strings.SplitN(string(m.Content), " ", 2)
		newDubsHandler(s, m, tmp[1], redisClient)
	case "rmdubs":
		tmp := strings.SplitN(string(m.Content), " ", 2)
		rmDubsHandler(s, m, tmp[1], redisClient)
	case "newlunch":
		tmp := strings.SplitN(string(m.Content), " ", 2)
		newLunchHandler(s, m, tmp[1], redisClient)
	case "rmlunch":
		tmp := strings.SplitN(string(m.Content), " ", 2)
		rmLunchHandler(s, m, tmp[1], redisClient)
	case "newbrb":
		tmp := strings.SplitN(string(m.Content), " ", 2)
		newBrbHandler(s, m, tmp[1], redisClient)
	case "rmbrb":
		tmp := strings.SplitN(string(m.Content), " ", 2)
		rmBrbHandler(s, m, tmp[1], redisClient)
	case "rmlore":
		rmHandler(s, m, content, "lore", redisClient)
	case "rek":
		tmp := strings.SplitN(string(m.Content), " ", 2)
		rekHandler(s, m, tmp[1], redisClient)
	case "rip":
		tmp := strings.SplitN(string(m.Content), " ", 2)
		ripHandler(s, m, tmp[1])
	case "summon":
		tmp := strings.SplitN(string(m.Content), " ", 2)
		summonHandler(s, m, tmp[1])
	case "gnu":
		tmp := strings.SplitN(string(m.Content), " ", 2)
		gnuHandler(s, m, tmp[1])
	case "created":
		attrHandler(s, m, "lore", content, "cdate", redisClient)
	case "owner":
		attrHandler(s, m, "lore", content, "owner", redisClient)
	case "removedby":
		attrHandler(s, m, "lore", content, "rmby", redisClient)
	case "removed":
		attrHandler(s, m, "lore", content, "rmdate", redisClient)

	}
}

func contentHandler(s *discordgo.Session, m *discordgo.MessageCreate, message, queryType string, redisClient *redis.Client) {
	loreTerm := strings.Replace(message, ".lore", "", -1)
	glogger.Debug.Printf("searching for: %s\n", loreTerm)

	content, err := loreCheck(loreTerm, queryType, redisClient)
	if err == nil {
		// lore found, print it
		s.ChannelMessageSend(m.ChannelID, content)
	} else {
		// lore not found, print error
		s.ChannelMessageSend(m.ChannelID, fmt.Sprintf("lore '%s' not found\n", loreTerm))
	}
}

func newHandler(s *discordgo.Session, m *discordgo.MessageCreate, message, newType string, redisClient *redis.Client) {
	// get nick from slack api
	nick := m.Author.Username

	err := loreNewString(nick, message, newType, redisClient)
	if err == nil {
		s.ChannelMessageSend(m.ChannelID, "entry created")
	} else {
		s.ChannelMessageSend(m.ChannelID, "entry could not be created")
	}
}

func newDubsHandler(s *discordgo.Session, m *discordgo.MessageCreate, newContent string, redisClient *redis.Client) {
	// prep content for dubsification
	// aka remove the '<' and '>' that slack adds
	newContent = strings.Replace(newContent, "<", "", -1)
	newContent = strings.Replace(newContent, ">", "", -1)

	err := redisClient.SAdd("index:dubs", newContent)
	if err != nil {
		s.ChannelMessageSend(m.ChannelID, "dubs checked")
	} else {
		s.ChannelMessageSend(m.ChannelID, "dubs check failed")
	}
}

func rmDubsHandler(s *discordgo.Session, m *discordgo.MessageCreate, rmContent string, redisClient *redis.Client) {
	// prep content for dubs removal
	// aka remove the '<' and '>' that slack adds
	rmContent = strings.Replace(rmContent, "<", "", -1)
	rmContent = strings.Replace(rmContent, ">", "", -1)

	err := redisClient.SRem("index:dubs", rmContent)
	if err != nil {
		s.ChannelMessageSend(m.ChannelID, "dubs unchecked")
	} else {
		s.ChannelMessageSend(m.ChannelID, "dubs check failed")
	}
}

func newLunchHandler(s *discordgo.Session, m *discordgo.MessageCreate, newContent string, redisClient *redis.Client) {
	// prep content for lunchification
	// aka remove the '<' and '>' that slack adds
	newContent = strings.Replace(newContent, "<", "", -1)
	newContent = strings.Replace(newContent, ">", "", -1)

	err := redisClient.SAdd("index:lunch", newContent)
	if err != nil {
		s.ChannelMessageSend(m.ChannelID, "lunch location added")
	} else {
		s.ChannelMessageSend(m.ChannelID, "lunch location addition failed")
	}
}

func rmLunchHandler(s *discordgo.Session, m *discordgo.MessageCreate, rmContent string, redisClient *redis.Client) {
	// prep content for lunch removal
	// aka remove the '<' and '>' that slack adds
	rmContent = strings.Replace(rmContent, "<", "", -1)
	rmContent = strings.Replace(rmContent, ">", "", -1)

	err := redisClient.SRem("index:lunch", rmContent)
	if err != nil {
		s.ChannelMessageSend(m.ChannelID, "lunch location removed")
	} else {
		s.ChannelMessageSend(m.ChannelID, "lunch location removal failed")
	}
}

func newBrbHandler(s *discordgo.Session, m *discordgo.MessageCreate, newContent string, redisClient *redis.Client) {
	// prep content for brbification
	// aka remove the '<' and '>' that slack adds
	newContent = strings.Replace(newContent, "<", "", -1)
	newContent = strings.Replace(newContent, ">", "", -1)

	err := redisClient.SAdd("index:brb", newContent)
	if err != nil {
		s.ChannelMessageSend(m.ChannelID, "tjancism added")
	} else {
		s.ChannelMessageSend(m.ChannelID, "tjancism addition failed")
	}
}

func rmBrbHandler(s *discordgo.Session, m *discordgo.MessageCreate, rmContent string, redisClient *redis.Client) {
	// prep content for brb removal
	// aka remove the '<' and '>' that slack adds
	rmContent = strings.Replace(rmContent, "<", "", -1)
	rmContent = strings.Replace(rmContent, ">", "", -1)

	err := redisClient.SRem("index:brb", rmContent)
	if err != nil {
		s.ChannelMessageSend(m.ChannelID, "tjancism removed")
	} else {
		s.ChannelMessageSend(m.ChannelID, "tjancism removal failed")
	}
}

func rmHandler(s *discordgo.Session, m *discordgo.MessageCreate, message, rmType string, redisClient *redis.Client) {
	nick := m.Author.Username

	err := loreRmString(nick, message, rmType, redisClient)
	if err == nil {
		s.ChannelMessageSend(m.ChannelID, "entry removed")
	} else {
		s.ChannelMessageSend(m.ChannelID, "entry could not be removed")
	}
}

func rekHandler(s *discordgo.Session, m *discordgo.MessageCreate, target string, redisClient *redis.Client) {
	rektBit, _ := redisClient.SRandMember("index:rekt").Result()
	rekFmt := fmt.Sprintf("%s %s", target, rektBit)
	s.ChannelMessageSend(m.ChannelID, rekFmt)
}

func ripHandler(s *discordgo.Session, m *discordgo.MessageCreate, target string) {
	// create a timestamp to use
	t := time.Now()
	tstamp := t.Format("01-02-2006")
	ripFmt := fmt.Sprintf("here lies %s, disrespected to death %s..", target, tstamp)
	s.ChannelMessageSend(m.ChannelID, ripFmt)
}

func summonHandler(s *discordgo.Session, m *discordgo.MessageCreate, target string) {
	sumFmt := fmt.Sprintf("つ◕_◕)つ %s つ◕_◕)つ", target)
	s.ChannelMessageSend(m.ChannelID, sumFmt)
}

func gnuHandler(s *discordgo.Session, m *discordgo.MessageCreate, target string) {
	gnuFmt := fmt.Sprintf("slaps the nonfree software out of %s's hands", target)
	s.ChannelMessageSend(m.ChannelID, gnuFmt)
}

func attrHandler(s *discordgo.Session, m *discordgo.MessageCreate, queryType, queryAttr, message string, redisClient *redis.Client) {
	query, err := loreAttrQuery(message, queryType, queryAttr, redisClient)
	if err == nil {
		s.ChannelMessageSend(m.ChannelID, query)
	} else {
		s.ChannelMessageSend(m.ChannelID, "attribute does not exist")
	}
}

func dHandler(s *discordgo.Session, m *discordgo.MessageCreate, num int) {
	if (num > 0) && (num < 99999999) {
		rand.Seed(time.Now().UnixNano())
		s.ChannelMessageSend(m.ChannelID, fmt.Sprintf("%d", rand.Intn(num)+1))
	} else {
		s.ChannelMessageSend(m.ChannelID, "na fam, we need a different number. stay lit tho")
	}
}
